/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.care.app.ui;

import com.care.alert_button.AlertButtonMonitor;
import com.care.app.AlreadyHaveMonitorAttachedException;
import com.care.app.ObserverManager;
import com.care.app.SmartMonitors;
import com.care.app.connector.*;
import com.care.app.observers.AlertButtonObserverImpl;
import com.care.app.observers.SmartTrackerObserverImpl;
import com.care.blood_pressure.BloodPressureMonitor;
import com.care.data_center.entities.Patient;
import com.care.heart_rate.HeartRateMonitor;
import com.care.smart_tracker.SmartTrackerMonitor;
import com.care.temperature_monitor.TemperatureMonitor;

import javax.swing.*;
import java.util.Arrays;

/**
 * @author wmespindula
 */
public class MonitorRegisterUI extends DisposableJFrame {
    private final Patient patient;
    private DisposableJFrame newMonitorUI;

    /**
     * Creates new form MainUI
     */
    public MonitorRegisterUI(Patient patient) {
        initComponents();

        this.patient = patient;

        Arrays.stream(SmartMonitors.values())
                .forEach(smartMonitor -> monitorTypeSelector.addItem(smartMonitor.getName()));
        refreshAvailableMonitorsByType(monitorTypeSelector.getSelectedItem());

    }

    private void refreshAvailableMonitorsByType(Object selectedItem) {
        SmartMonitors type = Arrays.stream(SmartMonitors.values()).filter(item -> item.getName().equals(selectedItem))
                .findFirst().get();
        switch (type) {
            case HEART_RATE:
                HeartRateConnector heartRateConnector = HeartRateConnector.getInstance();
                availableSensorsList.setListData(heartRateConnector.findAllNotUsed().stream()
                        .map(item -> item.getCode().toString()).toArray(String[]::new));
                break;
            case BLOOD_PRESSURE:
                BloodPressureConnector bloodPressureConnector = BloodPressureConnector.getInstance();
                availableSensorsList.setListData(bloodPressureConnector.findAllNotUsed().stream()
                        .map(item -> item.getCode().toString()).toArray(String[]::new));
                break;
            case SMART_TRACKER:
                SmartTrackerConnector smartTrackerConnector = SmartTrackerConnector.getInstance();
                availableSensorsList.setListData(smartTrackerConnector.findAllNotUsed().stream()
                        .map(item -> item.getCode().toString()).toArray(String[]::new));
                break;
            case TEMPERATURE_MONITOR:
                TemperatureConnector temperatureConnector = TemperatureConnector.getInstance();
                availableSensorsList.setListData(temperatureConnector.findAllNotUsed().stream()
                        .map(item -> item.getCode().toString()).toArray(String[]::new));
                break;
            case EMERGENCY_BUTTON:
                AlertButtonConnector alertButtonConnector = AlertButtonConnector.getInstance();
                availableSensorsList.setListData(alertButtonConnector.findAll().stream()
                        .map(item -> item.getCode().toString()).toArray(String[]::new));
                break;
            default:
                availableSensorsList.setListData(new String[]{});
                break;

        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        availableSensorsList = new javax.swing.JList<>();
        jLabel2 = new javax.swing.JLabel();
        monitorTypeSelector = new javax.swing.JComboBox<>();
        filler3 = new javax.swing.Box.Filler(new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 32767));
        addMonitorButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText("Adicionar Monitor à Paciente");
        jLabel1.setVerifyInputWhenFocusTarget(false);

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.PAGE_AXIS));

        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));
        jPanel1.add(jPanel2);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));
        jPanel1.add(jPanel3);

        availableSensorsList.setModel(new javax.swing.AbstractListModel<String>() {
            String[] strings = {"Item 1", "Item 2", "Item 3", "Item 4", "Item 5"};

            public int getSize() {
                return strings.length;
            }

            public String getElementAt(int i) {
                return strings[i];
            }
        });
        jScrollPane1.setViewportView(availableSensorsList);

        jLabel2.setText("Tipo de Sensor:");

        monitorTypeSelector.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                monitorTypeSelectorActionPerformed(evt);
            }
        });

        addMonitorButton.setText("Adicionar Monitor");
        addMonitorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addMonitorButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                        .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 267, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addGap(0, 0, Short.MAX_VALUE))
                                        .addGroup(layout.createSequentialGroup()
                                                .addGap(0, 0, Short.MAX_VALUE)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                        .addComponent(addMonitorButton)
                                                        .addGroup(layout.createSequentialGroup()
                                                                .addComponent(jLabel2)
                                                                .addGap(0, 0, 0)
                                                                .addComponent(filler3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                                .addGap(0, 0, 0)
                                                                .addComponent(monitorTypeSelector, javax.swing.GroupLayout.PREFERRED_SIZE, 148, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                                .addContainerGap())
        );
        layout.setVerticalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addGap(5, 5, 5)
                                                .addComponent(jLabel2))
                                        .addComponent(filler3, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(monitorTypeSelector, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(34, 34, 34)
                                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 351, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(addMonitorButton)
                                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void monitorTypeSelectorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_monitorTypeSelectorActionPerformed
        refreshAvailableMonitorsByType(monitorTypeSelector.getSelectedItem());
    }//GEN-LAST:event_monitorTypeSelectorActionPerformed

    private void addMonitorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addMonitorButtonActionPerformed

        SmartMonitors type = Arrays.stream(SmartMonitors.values()).filter(item -> item.getName().equals(monitorTypeSelector.getSelectedItem()))
                .findFirst().get();


        switch (type) {
            case HEART_RATE:
                HeartRateConnector heartRateConnector = HeartRateConnector.getInstance();
                Long heartRateMonitorValue = Long.parseLong(availableSensorsList.getSelectedValue());
                HeartRateMonitor heartRateMonitor = heartRateConnector.findByCode(heartRateMonitorValue).get();
                if (newMonitorUI == null || newMonitorUI.isDisposed()) {
                    newMonitorUI = new HeartRateRegisterUI(this, patient, heartRateMonitor);
                    newMonitorUI.setVisible(true);
                }
                break;


            case BLOOD_PRESSURE:
                BloodPressureConnector bloodPressureConnector = BloodPressureConnector.getInstance();
                Long bloodPressureMonitorValue = Long.parseLong(availableSensorsList.getSelectedValue());
                BloodPressureMonitor bloodPressureMonitor = bloodPressureConnector.
                        findByCode(bloodPressureMonitorValue).get();
                if (newMonitorUI == null || newMonitorUI.isDisposed()) {
                    newMonitorUI = new BloodPressureRegisterUI(this, patient, bloodPressureMonitor);
                    newMonitorUI.setVisible(true);
                }
                break;


            case SMART_TRACKER:
                SmartTrackerConnector smartTrackerConnector = SmartTrackerConnector.getInstance();
                Long smartTrackerMonitorValue = Long.parseLong(availableSensorsList.getSelectedValue());
                SmartTrackerMonitor smartTrackerMonitor = smartTrackerConnector.
                        getByCode(smartTrackerMonitorValue).get();
                registerSmartTracker(smartTrackerMonitor);
                break;

            case TEMPERATURE_MONITOR:
                TemperatureConnector temperatureConnector = TemperatureConnector.getInstance();
                Long temperatureMonitorValue = Long.parseLong(availableSensorsList.getSelectedValue());
                TemperatureMonitor temperatureMonitor = temperatureConnector.findByCode(temperatureMonitorValue).get();
                if (newMonitorUI == null || newMonitorUI.isDisposed()) {
                    newMonitorUI = new TemperatureMonitorRegisterUI(this, patient, temperatureMonitor);
                    newMonitorUI.setVisible(true);
                }

            case EMERGENCY_BUTTON:
                AlertButtonConnector alertConnector = AlertButtonConnector.getInstance();
                Long alertMonitorValue = Long.parseLong(availableSensorsList.getSelectedValue());
                AlertButtonMonitor alertButtonMonitor = alertConnector.findByCode(alertMonitorValue).get();
                registerEmergencyButton(alertButtonMonitor);
                break;

            default:
                availableSensorsList.setListData(new String[]{});
                break;

        }
    }//GEN-LAST:event_addMonitorButtonActionPerformed

    private void registerEmergencyButton(AlertButtonMonitor alertButtonMonitor) {
        AlertButtonObserverImpl alertButtonObserver = new AlertButtonObserverImpl(patient.getPatientId(), alertButtonMonitor.getCode());
        ObserverManager observerManager = ObserverManager.getInstance();

        try {
            observerManager.add(alertButtonObserver);

            AlertButtonConnector connector = AlertButtonConnector.getInstance();
            connector.attachPatientToMonitor(patient, alertButtonMonitor.getCode(),
                    alertButtonObserver);

            JOptionPane.showMessageDialog(this, "Monitor configurado ao paciente com sucesso!",
                    "Configurado!", JOptionPane.INFORMATION_MESSAGE);

        } catch (AlreadyHaveMonitorAttachedException e) {
            JOptionPane.showMessageDialog(this, "Botão de Emergência já está registrado ao paciente!");
            e.printStackTrace();
        }
        this.dispose();
    }

    private void registerSmartTracker(SmartTrackerMonitor smartTrackerMonitor) {
        SmartTrackerObserverImpl smartTrackerObserver = new SmartTrackerObserverImpl(patient.getPatientId(), smartTrackerMonitor.getCode());
        ObserverManager observerManager = ObserverManager.getInstance();

        try {
            observerManager.add(smartTrackerObserver);

            SmartTrackerConnector connector = SmartTrackerConnector.getInstance();
            connector.attachPatientToMonitor(patient, smartTrackerMonitor.getCode(),
                    smartTrackerObserver);

            JOptionPane.showMessageDialog(this, "Monitor configurado ao paciente com sucesso!",
                    "Configurado!", JOptionPane.INFORMATION_MESSAGE);

        } catch (AlreadyHaveMonitorAttachedException e) {
            JOptionPane.showMessageDialog(this, "Sensor de Pulso já está registrado ao paciente!");
            e.printStackTrace();
        }
        this.dispose();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addMonitorButton;
    private javax.swing.JList<String> availableSensorsList;
    private javax.swing.Box.Filler filler3;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JComboBox<String> monitorTypeSelector;
    // End of variables declaration//GEN-END:variables
}
